--[[
Name: FlexBar2 Core
Revision: $Rev: 30060 $
Author: Cncfanatics
Description: Action Bars manager Mod, Core file
Dependencies: AceLibrary, AceOO-2.0, AceOO-2.0, AceEvent-2.0, AceAddon-2.0,AceDB-2.0, AceDebug-2.0, AceHook-2.1, AceConsole-2.0, AceLocale-2.2, Dewdrop-2.0, Tablet-2.0, FuBarPlugin-2.0
]]
FlexBar2 = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceDB-2.0", "AceHook-2.1", "AceDebug-2.0", "AceEvent-2.0"); -- Note: not registerring AceEvent-2.0 here, but in every class, as the event handling methods are in there anyway.
FlexBar2.OO = AceLibrary("AceOO-2.0");
FlexBar2.BabbleSpell = AceLibrary("Babble-Spell-2.2");
FlexBar2.Aura = AceLibrary("SpecialEvents-Aura-2.0");
FlexBar2.PaintChips = AceLibrary("PaintChips-2.0");
-- FlexBar2.uiQuery = AceLibrary("uiQuery-1.0"); Currently disabled cuz its buggy

FlexBar2:RegisterDB("FlexBar2_DB");
FlexBar2:RegisterDefaults("profile",
{
	Button = {},
	Groups = {},
	Ticker = "global",
	GenButtons = {}, -- List of classnames to generate by default
	Colors = {
		Unlocked = {160,32,240}, --Purple
		Channeling = {0,0,180}, -- Blue
		AutoCast = {240,240,240}, -- Nearly White
		Casting = {180,180,0}, -- Yellow
		Buffed = {0,180,0}, -- Green
		DeBuffed = {180,0,0}, -- Red
		Equipped = {200,140,0}, -- Orange
		OutOfMana = {0,0,240}, -- Blue
		NotInRange = {200,0,0}, -- Red
		InCooldown = {160,160,160}, -- Grey
		UnusableItem = {160,160,160}, -- Gray
		UnusableAction = {160,160,160}, -- Gray
	},
	ColorPriority = {
		"Unlocked",
		"AutoCast",
		"Channeling",
		"Casting",
		"Buffed",
		"Debuffed",
		"Equipped",
	},
	StatePriority = {
		"InCooldown",
		"UnusableItem",
		"UnusableAction",
		"OutOfMana",
		"NotInRange",
	},
});

FlexBar2.Activate = {}; -- This table will be a table of functions that will each get run once when a class is initiated

FlexBar2.Class =  {}; -- Table containing all classes that are registered and will thus be loaded

FlexBar2.EventTable = {};

FlexBar2.Buttons = {}; -- Contains the objects of all buttons

FlexBar2.StartupScripts = {};

function FlexBar2:Startup()
	--[[
		runs with the ADDON_LOADED and launches all :Activate(); methods in all registered button objects
	--]]
	self:RegisterColors();
	for i, v in pairs(self.db.profile.GenButtons) do
		if(v == true) then
			self:Generate(i);
		end
	end
	for _, Func in ipairs(self.StartupScripts) do
		Func();
	end
end

function FlexBar2:AddStartupScript(Func)
	table.insert(self.StartupScripts, Func);
end

function FlexBar2:OnEnable()
	self:SetDebugging(false);
	self:Startup();
	self.SecureStateDriverFrame = CreateFrame("Frame", nil, UIParent, "SecureStateDriverTemplate");
	self.SecureAnchorButtonFrame = CreateFrame("Frame", nil, UIParent, "SecureAnchorButtonTemplate");
	self.SecureAnchorEnterFrame = CreateFrame("Frame", nil, UIParent, "SecureAnchorEnterTemplate");
	self.SecureAnchorUpDownFrame = CreateFrame("Frame", nil, UIParent, "SecureAnchorUpDownTemplate");
	self.SecurePieButtonFrame = CreateFrame("Frame", nil, UIParent, "SecurePieButtonTemplate");
	self.SecurePartyHeaderFrame = CreateFrame("Frame", nil, UIParent, "SecurePartyHeaderTemplate");
	self.SecureRaidGroupHeaderFrame = CreateFrame("Frame", nil, UIParent, "SecureRaidGroupHeaderTemplate");
	FlexBar2:ScheduleRepeatingEvent("FB2_UPDATE", 0.5);
end

function FlexBar2:RegisterColors()
	for n,v in pairs(self.db.profile.Colors) do
		FlexBar2.PaintChips:RegisterColor(n, unpack(v));
	end
end

function FlexBar2:OnProfileEnable()
	self:Startup();
	self:Login();
end

function FlexBar2:Generate(i, save)
	self:Debug("Generating " , i);
	local Mixin = self.OO.Class(unpack(self.Class));
	function Mixin.prototype:init(...)
		Mixin.super.prototype.init(self);
		for _, v in ipairs(FlexBar2.Activate) do
			v(self, ...);
		end
	end
	self.Buttons[i] = Mixin:new(i);
	if(save) then
		self.db.profile.GenButtons[tostring(i)] = true;
	end
end

function FlexBar2:Destroy(i)
	self:Debug("Destroying " , i);
	table.remove(self.db.profile.GenButtons, tostring(i));
end


function FlexBar2:RegisterHandler(reference)
	--[[
		Register a class with FlexBar2, it will be used in any buttons that are generated after it has been registered (make sure you register your class early !)
		There is also a second, optional argument that adds a function that will be executed each time a class is initiated
	--]]
		table.insert(self.Class, reference);
		if(type(reference.Activate) == "function") then
			table.insert(self.Activate, reference.Activate);
		end
		return true;
end


-- Utility functions used on many parts of the addon
function FlexBar2:CopyTable(t, NoFollowing)
	local x = {}
	for k,v in pairs(t) do
		if(type(v) == "table" and NoFollowing ~= true) then
			x[k] = self:CopyTable(v);
		else
			x[k] = v;
		end
		
	end
	setmetatable(x, getmetatable(t))
	return x
end

function FlexBar2:xor(a, b)
	return not (a ~= nil and a ~= false) == (b ~= nil and b ~= false);
end

function FlexBar2:ValueIsInTable(tbl, val)
	for n, v in pairs(tbl) do
		if(v == val) then
			return true;
		end
	end
	return false;
end

function FlexBar2:RemoveByValue(tbl, val)
	if(type(tbl) ~= "table") then return false; end
	for n, v in pairs(tbl) do
		if(v == val) then
			table.remove(tbl, n);
			return true;
		end
	end
	return false;
end

function FlexBar2:GetKeyByValue(tbl, val)
	for n, v in pairs(tbl) do
		if(v == val) then
			return n;
		end
	end
	return false;
end
